from django.conf import settings
from django.test import TestCase, Client
from django.urls import reverse

from model_bakery import baker


class LoginTestCase(TestCase):
    def setUp(self):
        self.staff_user = baker.make(
            'auth.User',
            username='user',
            user_roles__role='staff',
        )
        self.password = 'password'
        self.staff_user.set_password(self.password)
        self.staff_user.save()

        self.login_url = reverse('gas:login')

    def test_redirect(self):
        client = Client()

        # with next parametter redirects to custom url
        response = client.post(self.login_url, {
            'next': 'next_url',
            'username': self.staff_user.username,
            'password': self.password,
            'is_active': True,
        })
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, 'next_url')

        # with no next parametter redirects to default url
        response = client.post(self.login_url, {
            'username': self.staff_user.username,
            'password': self.password,
        })
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse('gas:index'))



from django.test import TestCase

from model_bakery import baker

from gas import models
from gas.sites import site


class UserRoleTestCase(TestCase):
    def test_str(self):
        user_role = baker.make('gas.UserRole')
        self.assertEqual(str(user_role), user_role.role)

    def test_description(self):
        user_role = baker.make('gas.UserRole')
        self.assertEqual(user_role.description, site.get_role_description(user_role.role))



from django.urls import path

from . import views


urlpatterns = [
    path('', views.UserList.as_view(), name="user_list"),
    path('create/', views.CreateUser.as_view(), name="user_create"),
    path('<int:pk>/edit/', views.UpdateUser.as_view(), name="user_update"),
    path('<int:pk>/change-password/', views.ChangePasswordUser.as_view(), name="user_change_password"),
    path('<int:pk>/delete/', views.DeleteUser.as_view(), name="user_delete"),
]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class GASConfig(AppConfig):
    name = 'gas'
    verbose_name = _('GAS')

    def ready(self):
        from .sites import site
        super().ready()
        site.autodiscover()



from rest_framework.authentication import SessionAuthentication
from rest_framework.permissions import BasePermission


class HasValidRole(BasePermission):
    def has_permission(self, request, view):
        user = request.user
        roles = set(view.roles)
        roles.add(view.base_role)
        access_denied = (
            not user.is_authenticated or (
                not user.is_superuser
                and user.user_roles.filter(role__in=roles).count() == 0
            )
        )
        return not access_denied


class GASAuthMixin:
    authentication_classes = [SessionAuthentication]
    permission_classes = [HasValidRole]
    base_role = 'admins'
    roles = set()



from django.contrib.auth.views import LoginView, PasswordChangeView
from django.shortcuts import resolve_url
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from django.views.generic import TemplateView

from gas.views import GASMixin

from gas import gas_settings


class GASLoginView(LoginView):
    template_name = "gas/login.html"

    def get_success_url(self):
        url = self.get_redirect_url()
        return url or resolve_url('gas:index')

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        css = gas_settings.MEDIA['css']
        js = gas_settings.MEDIA['js']
        if gas_settings.EXTRA_MEDIA:
            css = css + gas_settings.EXTRA_MEDIA.get('css', [])
            js = js + gas_settings.EXTRA_MEDIA.get('js', [])
        ctx.update({
            'logo_static_url': gas_settings.LOGO,
            'css': css,
            'js': js,
        })
        return ctx


class GASPasswordChangeView(GASMixin, PasswordChangeView):
    template_name = 'gas/base_form.html'
    success_url = reverse_lazy('gas:index')
    title = _('Change your password')
    success_message = _('Password changed.')


class Index(GASMixin, TemplateView):
    main_menu = 'index'
    template_name = "gas/index.html"
    roles = ('staff',)



import datetime
import json

from django.db.models import QuerySet
from django.utils.functional import Promise


class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M')
        if isinstance(obj, datetime.date):
            return obj.strftime('%Y-%m-%d')
        if isinstance(obj, QuerySet):
            return list(obj)
        if isinstance(obj, Promise):
            return str(obj)
        return json.JSONEncoder.default(self, obj)



import datetime
import json

from django.test import TestCase
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _

from model_bakery import baker

from gas import utils


class UtilsTestCase(TestCase):
    def test_jsonencoder(self):
        users = baker.make('auth.User', _quantity=3)
        now = datetime.datetime.now()
        today = datetime.date.today()
        data = {
            'now': now,
            'today': today,
            'users': User.objects.all().values_list('pk', flat=True),
            'lazy_string': _('lazy'),
        }

        dumped_data = json.dumps(data, cls=utils.JSONEncoder)
        recovered_data = json.loads(dumped_data)

        self.assertEqual(
            recovered_data['now'],
            now.strftime('%Y-%m-%d %H:%M')
        )
        self.assertEqual(
            recovered_data['today'],
            today.strftime('%Y-%m-%d')
        )
        self.assertEqual(
            len(recovered_data['users']),
            3,
        )
        self.assertEqual(
            set(recovered_data['users']),
            set(user.pk for user in users),
        )
        self.assertEqual(
            recovered_data['lazy_string'], 'lazy',
        )
